/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.fastkv

import io.fastkv.FastKVConfig.getExecutor
import java.util.Collections
import java.util.concurrent.atomic.AtomicBoolean

internal class ExternalExecutor {
    private val nameToTasks = Collections.synchronizedMap(HashMap<String, WrapperTask>())

    fun execute(name: String, task: CancelableTask) {
        val newTask = wrapTask(name, task)
        nameToTasks[name] = newTask
        getExecutor().execute(newTask)
    }

    private fun wrapTask(name: String, task: CancelableTask): WrapperTask {
        return object : WrapperTask(name, task) {
            override fun run() {
                try {
                    realTask.execute(canceled)
                } finally {
                    nameToTasks.remove(name)
                }
            }
        }
    }

    val isNotEmpty: Boolean
        get() = nameToTasks.isNotEmpty()

    /**
     * Try to mark task to be canceled.
     *
     * If there is a task with 'name' in progress, make it canceled an return true, return false
     * otherwise.
     */
    fun cancelTask(name: String): Boolean {
        val task = nameToTasks[name]
        if (task != null) {
            task.canceled.set(true)
            return true
        }
        return false
    }

    internal interface CancelableTask {
        fun execute(canceled: AtomicBoolean?)
    }

    private abstract class WrapperTask(val name: String, val realTask: CancelableTask) : Runnable {
        var canceled = AtomicBoolean()
    }
}
