/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.fastkv

import io.fastkv.FastKVConfig.IExecutor
import io.fastkv.interfaces.FastLogger
import java.util.concurrent.Executors

object FastKVConfig {
    @Volatile private var sExecutor: IExecutor? = null
    var internalLimit = 4096
        set(value) {
            if (value in 2048..0xFFFF) {
                field = value
            }
        }

    var sLogger: FastLogger = DefaultLogger() // 假设有一个 DefaultLogger 默认实现

    fun getExecutor(): IExecutor =
        sExecutor
            ?: synchronized(this) {
                sExecutor
                    ?: Executors.newCachedThreadPool().let { executor ->
                        IExecutor { runnable -> executor.execute(runnable) }.also { sExecutor = it }
                    }
            }

    fun setExecutor(executor: IExecutor?) {
        executor?.let { sExecutor = it }
    }

    fun interface IExecutor {
        fun execute(runnable: Runnable)
    }

    fun setLogger(logger: FastLogger) {
        sLogger = logger
    }
}
