/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.fastkv

import java.util.concurrent.Executor

/** Use for duplicate tasks. Only one task at most can wait to be executed. */
class LimitExecutor : Executor {
    private var mActive: Runnable? = null
    private var mWaiting: Runnable? = null

    @Synchronized
    override fun execute(r: Runnable) {
        if (mActive == null) {
            mActive = wrapTask(r)
            mActive?.let { FastKVConfig.getExecutor().execute(it) }
        } else {
            if (mWaiting == null) {
                mWaiting = wrapTask(r)
            }
        }
    }

    private fun wrapTask(r: Runnable): Runnable {
        return Runnable {
            try {
                r.run()
            } finally {
                scheduleNext()
            }
        }
    }

    @Synchronized
    private fun scheduleNext() {
        mActive = mWaiting
        mWaiting = null
        mActive?.let { FastKVConfig.getExecutor().execute(it) }
    }
}
