/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.fastkv

import io.fastkv.FastBuffer.Companion.getStringSize
import io.fastkv.FastBuffer.Companion.getVarint32Size
import io.fastkv.interfaces.FastEncoder

internal class StringSetEncoder private constructor() : FastEncoder<Set<String>> {
    override fun tag(): String {
        return "StringSet"
    }

    @Suppress("UNCHECKED_CAST")
    override fun encode(obj: Any): ByteArray {
        val src = obj as Set<String>
        return doEncode(src)
    }

    private fun doEncode(obj: Set<String>): ByteArray {
        if (obj.isEmpty()) {
            return ByteArray(0)
        }
        var count = 0
        val n = obj.size
        val sizeArray = IntArray(n)
        val strArray = arrayOfNulls<String>(n)
        for ((index, str) in obj.withIndex()) {
            val strSize = getStringSize(str)
            strArray[index] = str
            sizeArray[index] = strSize
            count += getVarint32Size(strSize) + strSize
        }
        val buffer = FastBuffer(count)
        for (i in 0 until n) {
            val size = sizeArray[i]
            buffer.putVarint32(size)
            if (size >= 0) {
                buffer.putString(strArray[i])
            }
        }
        return buffer.hb
    }

    override fun decode(bytes: ByteArray, offset: Int, length: Int): Set<String> {
        val set: MutableSet<String> = LinkedHashSet()
        if (length > 0) {
            val buffer = FastBuffer(bytes, offset)
            val limit = offset + length
            while (buffer.position < limit) {
                buffer.getString(buffer.varint32)?.let { set.add(it) }
            }
            require(buffer.position == limit) { "Invalid String set" }
        }
        return set
    }

    companion object {
        @JvmField val INSTANCE = StringSetEncoder()
    }
}
