/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.fastkv

import java.lang.ref.ReferenceQueue
import java.lang.ref.WeakReference

internal class WeakCache {
    private val cache: MutableMap<String, ValueReference> = HashMap()
    private val queue = ReferenceQueue<Any>()

    @Synchronized
    operator fun get(key: String): Any? {
        cleanQueue()
        val reference = cache[key]
        return reference?.get()
    }

    @Synchronized
    fun put(key: String, value: Any?) {
        cleanQueue()
        if (value != null) {
            val reference = cache[key]
            if (reference == null || reference.get() !== value) {
                cache[key] = ValueReference(key, value, queue)
            }
        }
    }

    @Synchronized
    fun remove(key: String) {
        cache.remove(key)
    }

    @Synchronized
    fun clear() {
        cache.clear()
        cleanQueue()
    }

    private fun cleanQueue() {
        var reference = queue.poll() as ValueReference
        while (reference != null) {
            val ref = cache[reference.key]
            if (ref != null && ref.get() == null) {
                cache.remove(reference.key)
            }
            reference = queue.poll() as ValueReference
        }
    }

    private class ValueReference(val key: String, value: Any?, q: ReferenceQueue<Any>) :
        WeakReference<Any?>(value, q as ReferenceQueue<in Any?>?)
}
